#!/usr/bin/env python

from setuptools import setup, find_packages

version = "1.10"

try:
    import pypandoc
    long_description = pypandoc.convert('README.md', 'rst')
except (IOError, ImportError):
    long_description = ""


setup(
    name="aioetcd3",
    version=version,
    author="gaopeiliang",
    author_email="964911957@qq.com",
    long_description=long_description,
    description="asyncio wrapper for etcd v3",
    license="Apache",
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Framework :: AsyncIO',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development',
        'Framework :: AsyncIO',
    ],
    url="https://github.com/gaopeiliang/aioetcd3",
    platforms=['any'],
    packages=find_packages(),
    python_requires='>=3.6',
    install_requires=[
        'aiogrpc>=1.4',
        'protobuf'
    ]
)



import unittest
import functools
import asyncio
from grpc import RpcError

from aioetcd3.client import client
from aioetcd3.help import range_all, range_prefix
from aioetcd3.watch import EVENT_TYPE_CREATE,EVENT_TYPE_DELETE,EVENT_TYPE_MODIFY,\
    CompactRevisonException, WatchException


def asynctest(f):
    @functools.wraps(f)
    def _f(self):
        return asyncio.get_event_loop().run_until_complete(f(self))

    return _f


class WatchTest(unittest.TestCase):
    def setUp(self):
        self.endpoints = "127.0.0.1:2379"
        self.client = client(endpoint=self.endpoints)
        self.tearDown()

    @asynctest
    async def test_watch_1(self):
        f1 = asyncio.get_event_loop().create_future()
        async def watch_1():
            i = 0
            async with self.client.watch_scope('/foo') as response:
                f1.set_result(None)
                async for event in response:
                    i = i + 1
                    if i == 1:
                        self.assertEqual(event.type, EVENT_TYPE_CREATE)
                        self.assertEqual(event.key, b'/foo')
                        self.assertEqual(event.value, b'foo')
                    elif i == 2:
                        self.assertEqual(event.type, EVENT_TYPE_MODIFY)
                        self.assertEqual(event.key, b'/foo')
                        self.assertEqual(event.value, b'foo1')
                    elif i == 3:
                        self.assertEqual(event.type, EVENT_TYPE_DELETE)
                        self.assertEqual(event.key, b'/foo')
                        # delete event has no value
                        # self.assertEqual(event.value, b'foo1')
                        break

        f2 = asyncio.get_event_loop().create_future()
        async def watch_2():
            i = 0
            async for event in self.client.watch('/foo', prev_kv=True, create_event=True):
                if event is None:
                    f2.set_result(None)
                    continue

                i = i + 1
                if i == 1:
                    self.assertEqual(event.type, EVENT_TYPE_CREATE)
                    self.assertEqual(event.key, b'/foo')
                    self.assertEqual(event.value, b'foo')
                elif i == 2:
                    self.assertEqual(event.type, EVENT_TYPE_MODIFY)
                    self.assertEqual(event.key, b'/foo')
                    self.assertEqual(event.value, b'foo1')
                    self.assertEqual(event.pre_value, b'foo')
                elif i == 3:
                    self.assertEqual(event.type, EVENT_TYPE_DELETE)
                    self.assertEqual(event.key, b'/foo')
                    # self.assertEqual(event.value, b'foo1')
                    break

        f3 = asyncio.get_event_loop().create_future()
        async def watch_3():
            i = 0
            async for event in self.client.watch('/foo', prev_kv=True, noput=True, create_event=True):
                if event is None:
                    f3.set_result(None)
                    continue

                i = i + 1
                if i == 1:
                    self.assertEqual(event.type, EVENT_TYPE_DELETE)
                    self.assertEqual(event.key, b'/foo')
                    # self.assertEqual(event.value, b'foo1')
                    break

        f4 = asyncio.get_event_loop().create_future()
        async def watch_4():
            i = 0
            async for event in self.client.watch('/foo', prev_kv=True, nodelete=True, create_event=True):
                if event is None:
                    f4.set_result(None)
                    continue

                i = i + 1
                if i == 1:
                    self.assertEqual(event.type, EVENT_TYPE_CREATE)
                    self.assertEqual(event.key, b'/foo')
                    self.assertEqual(event.value, b'foo')
                elif i == 2:
                    self.assertEqual(event.type, EVENT_TYPE_MODIFY)
                    self.assertEqual(event.key, b'/foo')
                    self.assertEqual(event.value, b'foo1')
                    self.assertEqual(event.pre_value, b'foo')
                    break

        w1 = asyncio.ensure_future(watch_1())
        w2 = asyncio.ensure_future(watch_2())
        w3 = asyncio.ensure_future(watch_3())
        w4 = asyncio.ensure_future(watch_4())

        await asyncio.wait_for(asyncio.wait([f1, f2, f3, f4]), 2)

        await self.client.put('/foo', 'foo')
        await self.client.put('/foo', 'foo1')
        await self.client.delete('/foo')

        done, pending = await asyncio.wait([w1, w2, w3, w4], timeout=20)
        for t in done:
            t.result()

    @asynctest
    async def test_watch_reconnect(self):
        f1 = asyncio.get_event_loop().create_future()
        f2 = asyncio.get_event_loop().create_future()
        async def watch_1():
            i = 0
            async with self.client.watch_scope('/foo') as response:
                f1.set_result(None)
                async for event in response:
                    i = i + 1
                    if i == 1:
                        self.assertEqual(event.type, EVENT_TYPE_CREATE)
                        self.assertEqual(event.key, b'/foo')
                        self.assertEqual(event.value, b'foo')
                        f2.set_result(None)
                    elif i == 2:
                        self.assertEqual(event.type, EVENT_TYPE_MODIFY)
                        self.assertEqual(event.key, b'/foo')
                        self.assertEqual(event.value, b'foo1')
                    elif i == 3:
                        self.assertEqual(event.type, EVENT_TYPE_DELETE)
                        self.assertEqual(event.key, b'/foo')
                        # delete event has no value
                        # self.assertEqual(event.value, b'foo1')
                        break
        t1 = asyncio.ensure_future(watch_1())
        await f1
        await self.client.put('/foo', 'foo')
        await f2
        self.client.update_server_list(self.endpoints)
        await self.client.put('/foo', 'foo1')
        await self.client.delete('/foo')
        await t1
    
    @asynctest
    async def test_watch_create_cancel(self):
        async def watch_1():
            async with self.client.watch_scope('/foo') as _:
                pass
        async def watch_2():
            async with self.client.watch_scope('/foo') as _:
                await asyncio.sleep(5)
        for _ in range(0, 5):
            watches = [asyncio.ensure_future(watch_1() if i % 2 else watch_2()) for i in range(0, 200)]
            await asyncio.sleep(1)
            for w in watches[::3]:
                w.cancel()
            self.client.update_server_list(self.endpoints)
            await asyncio.sleep(0.01)
            for w in watches[1::3]:
                w.cancel()
            await asyncio.sleep(0.3)
            for w in watches[2::3]:
                w.cancel()
            await asyncio.wait_for(asyncio.wait(watches), 3)
            results = await asyncio.gather(*watches, return_exceptions=True)
            print("Finished:", len([r for r in results if r is None]), "Cancelled:", len([r for r in results if r is not None]))
            self.assertIsNotNone(self.client._watch_task_running)
        await asyncio.sleep(3)
        self.assertIsNone(self.client._watch_task_running)
    
    @asynctest
    async def test_batch_events(self):
        f1 = asyncio.get_event_loop().create_future()
        f2 = asyncio.get_event_loop().create_future()
        def _check_event(e, criterias):
            if criterias[0]:
                self.assertEqual(e.type, criterias[0])
            if criterias[1]:
                self.assertEqual(e.key, criterias[1])
            if criterias[2]:
                self.assertEqual(e.value, criterias[2])
        async def watch_1():
            asserts = [(EVENT_TYPE_CREATE, b'/foo/1', b'1'),
                       (EVENT_TYPE_CREATE, b'/foo/2', b'2'),
                       (EVENT_TYPE_MODIFY, b'/foo/1', b'2'),
                       (EVENT_TYPE_MODIFY, b'/foo/2', b'3'),
                       (EVENT_TYPE_DELETE, b'/foo/1',None),
                       (EVENT_TYPE_DELETE, b'/foo/2',None)]
            async with self.client.watch_scope(range_prefix('/foo/')) as response:
                f1.set_result(None)
                async for e in response:
                    _check_event(e, asserts.pop(0))
                    if not asserts:
                        break
        
        async def watch_2():
            asserts = [((EVENT_TYPE_CREATE, b'/foo/1', b'1'),
                       (EVENT_TYPE_CREATE, b'/foo/2', b'2'),),
                       ((EVENT_TYPE_MODIFY, b'/foo/1', b'2'),),
                       ((EVENT_TYPE_MODIFY, b'/foo/2', b'3'),),
                       ((EVENT_TYPE_DELETE, b'/foo/1',None),
                       (EVENT_TYPE_DELETE, b'/foo/2',None))]
            async with self.client.watch_scope(range_prefix('/foo/'), batch_events=True)\
                    as response:
                f2.set_result(None)
                async for es in response:
                    batch = asserts.pop(0)
                    self.assertEqual(len(es), len(batch))
                    for e, a in zip(es, batch):
                        _check_event(e, a)
                    if not asserts:
                        break
        
        t1 = asyncio.ensure_future(watch_1())
        t2 = asyncio.ensure_future(watch_2())
        await asyncio.wait_for(asyncio.wait([f1,f2]), 2)
        self.assertTrue((await self.client.txn([],[self.client.put.txn('/foo/1', '1'),
                                  self.client.put.txn('/foo/2', '2')],[]))[0])
        await self.client.put('/foo/1', '2')
        await self.client.put('/foo/2', '3')
        self.assertTrue((await self.client.txn([],[self.client.delete.txn('/foo/1'),
                                  self.client.delete.txn('/foo/2')],[]))[0])        
        await asyncio.gather(t1, t2)
    
    @asynctest
    async def test_compact_revision(self):
        await self.client.put('/foo', '1')
        first_revision = self.client.last_response_info.revision
        await self.client.put('/foo', '2')
        await self.client.put('/foo', '3')
        await self.client.put('/foo', '4')
        await self.client.put('/foo', '5')
        compact_revision = self.client.last_response_info.revision
        await self.client.compact(compact_revision, True)
        async def watch_1():
                async with self.client.watch_scope('/foo', start_revision=first_revision) as response:
                    with self.assertRaises(CompactRevisonException) as cm:
                        async for e in response:
                            raise ValueError("Not raised")
                    self.assertEqual(cm.exception.revision, compact_revision)
        
        async def watch_2():
            async with self.client.watch_scope('/foo', ignore_compact=True, start_revision=first_revision) as responses:
                async for e in responses:
                    self.assertEqual(e.type, EVENT_TYPE_MODIFY)
                    self.assertEqual(e.key, b'/foo')
                    self.assertEqual(e.value, b'5')
                    self.assertEqual(e.revision, compact_revision)
                    break
        await watch_1()
        await watch_2()

    @asynctest
    async def test_watch_exception(self):
        f1 = asyncio.get_event_loop().create_future()
        f2 = asyncio.get_event_loop().create_future()
        async def watch_1():
            i = 0
            async with self.client.watch_scope('/foo') as response:
                f1.set_result(None)
                with self.assertRaises(WatchException):
                    async for event in response:
                        i = i + 1
                        if i == 1:
                            self.assertEqual(event.type, EVENT_TYPE_CREATE)
                            self.assertEqual(event.key, b'/foo')
                            self.assertEqual(event.value, b'foo')
                            f2.set_result(None)
                        elif i == 2:
                            raise ValueError("Not raised")
        f3 = asyncio.get_event_loop().create_future()
        f4 = asyncio.get_event_loop().create_future()
        async def watch_2():
            i = 0
            async with self.client.watch_scope('/foo', always_reconnect=True) as response:
                f3.set_result(None)
                async for event in response:
                    i = i + 1
                    if i == 1:
                        self.assertEqual(event.type, EVENT_TYPE_CREATE)
                        self.assertEqual(event.key, b'/foo')
                        self.assertEqual(event.value, b'foo')
                        f4.set_result(None)
                    elif i == 2:
                        self.assertEqual(event.type, EVENT_TYPE_MODIFY)
                        self.assertEqual(event.key, b'/foo')
                        self.assertEqual(event.value, b'foo1')
                    elif i == 3:
                        self.assertEqual(event.type, EVENT_TYPE_DELETE)
                        self.assertEqual(event.key, b'/foo')
                        # delete event has no value
                        # self.assertEqual(event.value, b'foo1')
                        break
        t1 = asyncio.ensure_future(watch_1())
        t2 = asyncio.ensure_future(watch_2())
        await f1
        await f3
        await self.client.put('/foo', 'foo')
        await f2
        await f4
        fake_endpoints = 'ipv4:///127.0.0.1:49999'
        self.client.update_server_list(fake_endpoints)
        await asyncio.sleep(2)
        self.client.update_server_list(self.endpoints)
        await self.client.put('/foo', 'foo1')
        await self.client.delete('/foo')
        await t1
        await t2
    
    @asynctest
    async def tearDown(self):
        await self.client.delete(range_all())


if __name__ == '__main__':
    unittest.main()



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from aioetcd3._etcdv3 import rpc_pb2 as rpc__pb2


class KVStub(object):
  """for grpc-gateway

  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.Range = channel.unary_unary(
        '/etcdserverpb.KV/Range',
        request_serializer=rpc__pb2.RangeRequest.SerializeToString,
        response_deserializer=rpc__pb2.RangeResponse.FromString,
        )
    self.Put = channel.unary_unary(
        '/etcdserverpb.KV/Put',
        request_serializer=rpc__pb2.PutRequest.SerializeToString,
        response_deserializer=rpc__pb2.PutResponse.FromString,
        )
    self.DeleteRange = channel.unary_unary(
        '/etcdserverpb.KV/DeleteRange',
        request_serializer=rpc__pb2.DeleteRangeRequest.SerializeToString,
        response_deserializer=rpc__pb2.DeleteRangeResponse.FromString,
        )
    self.Txn = channel.unary_unary(
        '/etcdserverpb.KV/Txn',
        request_serializer=rpc__pb2.TxnRequest.SerializeToString,
        response_deserializer=rpc__pb2.TxnResponse.FromString,
        )
    self.Compact = channel.unary_unary(
        '/etcdserverpb.KV/Compact',
        request_serializer=rpc__pb2.CompactionRequest.SerializeToString,
        response_deserializer=rpc__pb2.CompactionResponse.FromString,
        )


class KVServicer(object):
  """for grpc-gateway

  """

  def Range(self, request, context):
    """Range gets the keys in the range from the key-value store.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Put(self, request, context):
    """Put puts the given key into the key-value store.
    A put request increments the revision of the key-value store
    and generates one event in the event history.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DeleteRange(self, request, context):
    """DeleteRange deletes the given range from the key-value store.
    A delete request increments the revision of the key-value store
    and generates a delete event in the event history for every deleted key.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Txn(self, request, context):
    """Txn processes multiple requests in a single transaction.
    A txn request increments the revision of the key-value store
    and generates events with the same revision for every completed request.
    It is not allowed to modify the same key several times within one txn.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Compact(self, request, context):
    """Compact compacts the event history in the etcd key-value store. The key-value
    store should be periodically compacted or the event history will continue to grow
    indefinitely.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_KVServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'Range': grpc.unary_unary_rpc_method_handler(
          servicer.Range,
          request_deserializer=rpc__pb2.RangeRequest.FromString,
          response_serializer=rpc__pb2.RangeResponse.SerializeToString,
      ),
      'Put': grpc.unary_unary_rpc_method_handler(
          servicer.Put,
          request_deserializer=rpc__pb2.PutRequest.FromString,
          response_serializer=rpc__pb2.PutResponse.SerializeToString,
      ),
      'DeleteRange': grpc.unary_unary_rpc_method_handler(
          servicer.DeleteRange,
          request_deserializer=rpc__pb2.DeleteRangeRequest.FromString,
          response_serializer=rpc__pb2.DeleteRangeResponse.SerializeToString,
      ),
      'Txn': grpc.unary_unary_rpc_method_handler(
          servicer.Txn,
          request_deserializer=rpc__pb2.TxnRequest.FromString,
          response_serializer=rpc__pb2.TxnResponse.SerializeToString,
      ),
      'Compact': grpc.unary_unary_rpc_method_handler(
          servicer.Compact,
          request_deserializer=rpc__pb2.CompactionRequest.FromString,
          response_serializer=rpc__pb2.CompactionResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'etcdserverpb.KV', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))


class WatchStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.Watch = channel.stream_stream(
        '/etcdserverpb.Watch/Watch',
        request_serializer=rpc__pb2.WatchRequest.SerializeToString,
        response_deserializer=rpc__pb2.WatchResponse.FromString,
        )


class WatchServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def Watch(self, request_iterator, context):
    """Watch watches for events happening or that have happened. Both input and output
    are streams; the input stream is for creating and canceling watchers and the output
    stream sends events. One watch RPC can watch on multiple key ranges, streaming events
    for several watches at once. The entire event history can be watched starting from the
    last compaction revision.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_WatchServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'Watch': grpc.stream_stream_rpc_method_handler(
          servicer.Watch,
          request_deserializer=rpc__pb2.WatchRequest.FromString,
          response_serializer=rpc__pb2.WatchResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'etcdserverpb.Watch', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))


class LeaseStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.LeaseGrant = channel.unary_unary(
        '/etcdserverpb.Lease/LeaseGrant',
        request_serializer=rpc__pb2.LeaseGrantRequest.SerializeToString,
        response_deserializer=rpc__pb2.LeaseGrantResponse.FromString,
        )
    self.LeaseRevoke = channel.unary_unary(
        '/etcdserverpb.Lease/LeaseRevoke',
        request_serializer=rpc__pb2.LeaseRevokeRequest.SerializeToString,
        response_deserializer=rpc__pb2.LeaseRevokeResponse.FromString,
        )
    self.LeaseKeepAlive = channel.stream_stream(
        '/etcdserverpb.Lease/LeaseKeepAlive',
        request_serializer=rpc__pb2.LeaseKeepAliveRequest.SerializeToString,
        response_deserializer=rpc__pb2.LeaseKeepAliveResponse.FromString,
        )
    self.LeaseTimeToLive = channel.unary_unary(
        '/etcdserverpb.Lease/LeaseTimeToLive',
        request_serializer=rpc__pb2.LeaseTimeToLiveRequest.SerializeToString,
        response_deserializer=rpc__pb2.LeaseTimeToLiveResponse.FromString,
        )


class LeaseServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def LeaseGrant(self, request, context):
    """LeaseGrant creates a lease which expires if the server does not receive a keepAlive
    within a given time to live period. All keys attached to the lease will be expired and
    deleted if the lease expires. Each expired key generates a delete event in the event history.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def LeaseRevoke(self, request, context):
    """LeaseRevoke revokes a lease. All keys attached to the lease will expire and be deleted.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def LeaseKeepAlive(self, request_iterator, context):
    """LeaseKeepAlive keeps the lease alive by streaming keep alive requests from the client
    to the server and streaming keep alive responses from the server to the client.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def LeaseTimeToLive(self, request, context):
    """LeaseTimeToLive retrieves lease information.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_LeaseServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'LeaseGrant': grpc.unary_unary_rpc_method_handler(
          servicer.LeaseGrant,
          request_deserializer=rpc__pb2.LeaseGrantRequest.FromString,
          response_serializer=rpc__pb2.LeaseGrantResponse.SerializeToString,
      ),
      'LeaseRevoke': grpc.unary_unary_rpc_method_handler(
          servicer.LeaseRevoke,
          request_deserializer=rpc__pb2.LeaseRevokeRequest.FromString,
          response_serializer=rpc__pb2.LeaseRevokeResponse.SerializeToString,
      ),
      'LeaseKeepAlive': grpc.stream_stream_rpc_method_handler(
          servicer.LeaseKeepAlive,
          request_deserializer=rpc__pb2.LeaseKeepAliveRequest.FromString,
          response_serializer=rpc__pb2.LeaseKeepAliveResponse.SerializeToString,
      ),
      'LeaseTimeToLive': grpc.unary_unary_rpc_method_handler(
          servicer.LeaseTimeToLive,
          request_deserializer=rpc__pb2.LeaseTimeToLiveRequest.FromString,
          response_serializer=rpc__pb2.LeaseTimeToLiveResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'etcdserverpb.Lease', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))


class ClusterStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.MemberAdd = channel.unary_unary(
        '/etcdserverpb.Cluster/MemberAdd',
        request_serializer=rpc__pb2.MemberAddRequest.SerializeToString,
        response_deserializer=rpc__pb2.MemberAddResponse.FromString,
        )
    self.MemberRemove = channel.unary_unary(
        '/etcdserverpb.Cluster/MemberRemove',
        request_serializer=rpc__pb2.MemberRemoveRequest.SerializeToString,
        response_deserializer=rpc__pb2.MemberRemoveResponse.FromString,
        )
    self.MemberUpdate = channel.unary_unary(
        '/etcdserverpb.Cluster/MemberUpdate',
        request_serializer=rpc__pb2.MemberUpdateRequest.SerializeToString,
        response_deserializer=rpc__pb2.MemberUpdateResponse.FromString,
        )
    self.MemberList = channel.unary_unary(
        '/etcdserverpb.Cluster/MemberList',
        request_serializer=rpc__pb2.MemberListRequest.SerializeToString,
        response_deserializer=rpc__pb2.MemberListResponse.FromString,
        )


class ClusterServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def MemberAdd(self, request, context):
    """MemberAdd adds a member into the cluster.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def MemberRemove(self, request, context):
    """MemberRemove removes an existing member from the cluster.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def MemberUpdate(self, request, context):
    """MemberUpdate updates the member configuration.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def MemberList(self, request, context):
    """MemberList lists all the members in the cluster.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_ClusterServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'MemberAdd': grpc.unary_unary_rpc_method_handler(
          servicer.MemberAdd,
          request_deserializer=rpc__pb2.MemberAddRequest.FromString,
          response_serializer=rpc__pb2.MemberAddResponse.SerializeToString,
      ),
      'MemberRemove': grpc.unary_unary_rpc_method_handler(
          servicer.MemberRemove,
          request_deserializer=rpc__pb2.MemberRemoveRequest.FromString,
          response_serializer=rpc__pb2.MemberRemoveResponse.SerializeToString,
      ),
      'MemberUpdate': grpc.unary_unary_rpc_method_handler(
          servicer.MemberUpdate,
          request_deserializer=rpc__pb2.MemberUpdateRequest.FromString,
          response_serializer=rpc__pb2.MemberUpdateResponse.SerializeToString,
      ),
      'MemberList': grpc.unary_unary_rpc_method_handler(
          servicer.MemberList,
          request_deserializer=rpc__pb2.MemberListRequest.FromString,
          response_serializer=rpc__pb2.MemberListResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'etcdserverpb.Cluster', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))


class MaintenanceStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.Alarm = channel.unary_unary(
        '/etcdserverpb.Maintenance/Alarm',
        request_serializer=rpc__pb2.AlarmRequest.SerializeToString,
        response_deserializer=rpc__pb2.AlarmResponse.FromString,
        )
    self.Status = channel.unary_unary(
        '/etcdserverpb.Maintenance/Status',
        request_serializer=rpc__pb2.StatusRequest.SerializeToString,
        response_deserializer=rpc__pb2.StatusResponse.FromString,
        )
    self.Defragment = channel.unary_unary(
        '/etcdserverpb.Maintenance/Defragment',
        request_serializer=rpc__pb2.DefragmentRequest.SerializeToString,
        response_deserializer=rpc__pb2.DefragmentResponse.FromString,
        )
    self.Hash = channel.unary_unary(
        '/etcdserverpb.Maintenance/Hash',
        request_serializer=rpc__pb2.HashRequest.SerializeToString,
        response_deserializer=rpc__pb2.HashResponse.FromString,
        )
    self.HashKV = channel.unary_unary(
        '/etcdserverpb.Maintenance/HashKV',
        request_serializer=rpc__pb2.HashKVRequest.SerializeToString,
        response_deserializer=rpc__pb2.HashKVResponse.FromString,
        )
    self.Snapshot = channel.unary_stream(
        '/etcdserverpb.Maintenance/Snapshot',
        request_serializer=rpc__pb2.SnapshotRequest.SerializeToString,
        response_deserializer=rpc__pb2.SnapshotResponse.FromString,
        )
    self.MoveLeader = channel.unary_unary(
        '/etcdserverpb.Maintenance/MoveLeader',
        request_serializer=rpc__pb2.MoveLeaderRequest.SerializeToString,
        response_deserializer=rpc__pb2.MoveLeaderResponse.FromString,
        )


class MaintenanceServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def Alarm(self, request, context):
    """Alarm activates, deactivates, and queries alarms regarding cluster health.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Status(self, request, context):
    """Status gets the status of the member.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Defragment(self, request, context):
    """Defragment defragments a member's backend database to recover storage space.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Hash(self, request, context):
    """Hash computes the hash of the KV's backend.
    This is designed for testing; do not use this in production when there
    are ongoing transactions.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def HashKV(self, request, context):
    """HashKV computes the hash of all MVCC keys up to a given revision. 
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Snapshot(self, request, context):
    """Snapshot sends a snapshot of the entire backend from a member over a stream to a client.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def MoveLeader(self, request, context):
    """MoveLeader requests current leader node to transfer its leadership to transferee.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_MaintenanceServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'Alarm': grpc.unary_unary_rpc_method_handler(
          servicer.Alarm,
          request_deserializer=rpc__pb2.AlarmRequest.FromString,
          response_serializer=rpc__pb2.AlarmResponse.SerializeToString,
      ),
      'Status': grpc.unary_unary_rpc_method_handler(
          servicer.Status,
          request_deserializer=rpc__pb2.StatusRequest.FromString,
          response_serializer=rpc__pb2.StatusResponse.SerializeToString,
      ),
      'Defragment': grpc.unary_unary_rpc_method_handler(
          servicer.Defragment,
          request_deserializer=rpc__pb2.DefragmentRequest.FromString,
          response_serializer=rpc__pb2.DefragmentResponse.SerializeToString,
      ),
      'Hash': grpc.unary_unary_rpc_method_handler(
          servicer.Hash,
          request_deserializer=rpc__pb2.HashRequest.FromString,
          response_serializer=rpc__pb2.HashResponse.SerializeToString,
      ),
      'HashKV': grpc.unary_unary_rpc_method_handler(
          servicer.HashKV,
          request_deserializer=rpc__pb2.HashKVRequest.FromString,
          response_serializer=rpc__pb2.HashKVResponse.SerializeToString,
      ),
      'Snapshot': grpc.unary_stream_rpc_method_handler(
          servicer.Snapshot,
          request_deserializer=rpc__pb2.SnapshotRequest.FromString,
          response_serializer=rpc__pb2.SnapshotResponse.SerializeToString,
      ),
      'MoveLeader': grpc.unary_unary_rpc_method_handler(
          servicer.MoveLeader,
          request_deserializer=rpc__pb2.MoveLeaderRequest.FromString,
          response_serializer=rpc__pb2.MoveLeaderResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'etcdserverpb.Maintenance', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))


class AuthStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.AuthEnable = channel.unary_unary(
        '/etcdserverpb.Auth/AuthEnable',
        request_serializer=rpc__pb2.AuthEnableRequest.SerializeToString,
        response_deserializer=rpc__pb2.AuthEnableResponse.FromString,
        )
    self.AuthDisable = channel.unary_unary(
        '/etcdserverpb.Auth/AuthDisable',
        request_serializer=rpc__pb2.AuthDisableRequest.SerializeToString,
        response_deserializer=rpc__pb2.AuthDisableResponse.FromString,
        )
    self.Authenticate = channel.unary_unary(
        '/etcdserverpb.Auth/Authenticate',
        request_serializer=rpc__pb2.AuthenticateRequest.SerializeToString,
        response_deserializer=rpc__pb2.AuthenticateResponse.FromString,
        )
    self.UserAdd = channel.unary_unary(
        '/etcdserverpb.Auth/UserAdd',
        request_serializer=rpc__pb2.AuthUserAddRequest.SerializeToString,
        response_deserializer=rpc__pb2.AuthUserAddResponse.FromString,
        )
    self.UserGet = channel.unary_unary(
        '/etcdserverpb.Auth/UserGet',
        request_serializer=rpc__pb2.AuthUserGetRequest.SerializeToString,
        response_deserializer=rpc__pb2.AuthUserGetResponse.FromString,
        )
    self.UserList = channel.unary_unary(
        '/etcdserverpb.Auth/UserList',
        request_serializer=rpc__pb2.AuthUserListRequest.SerializeToString,
        response_deserializer=rpc__pb2.AuthUserListResponse.FromString,
        )
    self.UserDelete = channel.unary_unary(
        '/etcdserverpb.Auth/UserDelete',
        request_serializer=rpc__pb2.AuthUserDeleteRequest.SerializeToString,
        response_deserializer=rpc__pb2.AuthUserDeleteResponse.FromString,
        )
    self.UserChangePassword = channel.unary_unary(
        '/etcdserverpb.Auth/UserChangePassword',
        request_serializer=rpc__pb2.AuthUserChangePasswordRequest.SerializeToString,
        response_deserializer=rpc__pb2.AuthUserChangePasswordResponse.FromString,
        )
    self.UserGrantRole = channel.unary_unary(
        '/etcdserverpb.Auth/UserGrantRole',
        request_serializer=rpc__pb2.AuthUserGrantRoleRequest.SerializeToString,
        response_deserializer=rpc__pb2.AuthUserGrantRoleResponse.FromString,
        )
    self.UserRevokeRole = channel.unary_unary(
        '/etcdserverpb.Auth/UserRevokeRole',
        request_serializer=rpc__pb2.AuthUserRevokeRoleRequest.SerializeToString,
        response_deserializer=rpc__pb2.AuthUserRevokeRoleResponse.FromString,
        )
    self.RoleAdd = channel.unary_unary(
        '/etcdserverpb.Auth/RoleAdd',
        request_serializer=rpc__pb2.AuthRoleAddRequest.SerializeToString,
        response_deserializer=rpc__pb2.AuthRoleAddResponse.FromString,
        )
    self.RoleGet = channel.unary_unary(
        '/etcdserverpb.Auth/RoleGet',
        request_serializer=rpc__pb2.AuthRoleGetRequest.SerializeToString,
        response_deserializer=rpc__pb2.AuthRoleGetResponse.FromString,
        )
    self.RoleList = channel.unary_unary(
        '/etcdserverpb.Auth/RoleList',
        request_serializer=rpc__pb2.AuthRoleListRequest.SerializeToString,
        response_deserializer=rpc__pb2.AuthRoleListResponse.FromString,
        )
    self.RoleDelete = channel.unary_unary(
        '/etcdserverpb.Auth/RoleDelete',
        request_serializer=rpc__pb2.AuthRoleDeleteRequest.SerializeToString,
        response_deserializer=rpc__pb2.AuthRoleDeleteResponse.FromString,
        )
    self.RoleGrantPermission = channel.unary_unary(
        '/etcdserverpb.Auth/RoleGrantPermission',
        request_serializer=rpc__pb2.AuthRoleGrantPermissionRequest.SerializeToString,
        response_deserializer=rpc__pb2.AuthRoleGrantPermissionResponse.FromString,
        )
    self.RoleRevokePermission = channel.unary_unary(
        '/etcdserverpb.Auth/RoleRevokePermission',
        request_serializer=rpc__pb2.AuthRoleRevokePermissionRequest.SerializeToString,
        response_deserializer=rpc__pb2.AuthRoleRevokePermissionResponse.FromString,
        )


class AuthServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def AuthEnable(self, request, context):
    """AuthEnable enables authentication.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def AuthDisable(self, request, context):
    """AuthDisable disables authentication.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Authenticate(self, request, context):
    """Authenticate processes an authenticate request.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def UserAdd(self, request, context):
    """UserAdd adds a new user.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def UserGet(self, request, context):
    """UserGet gets detailed user information.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def UserList(self, request, context):
    """UserList gets a list of all users.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def UserDelete(self, request, context):
    """UserDelete deletes a specified user.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def UserChangePassword(self, request, context):
    """UserChangePassword changes the password of a specified user.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def UserGrantRole(self, request, context):
    """UserGrant grants a role to a specified user.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def UserRevokeRole(self, request, context):
    """UserRevokeRole revokes a role of specified user.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def RoleAdd(self, request, context):
    """RoleAdd adds a new role.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def RoleGet(self, request, context):
    """RoleGet gets detailed role information.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def RoleList(self, request, context):
    """RoleList gets lists of all roles.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def RoleDelete(self, request, context):
    """RoleDelete deletes a specified role.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def RoleGrantPermission(self, request, context):
    """RoleGrantPermission grants a permission of a specified key or range to a specified role.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def RoleRevokePermission(self, request, context):
    """RoleRevokePermission revokes a key or range permission of a specified role.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_AuthServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'AuthEnable': grpc.unary_unary_rpc_method_handler(
          servicer.AuthEnable,
          request_deserializer=rpc__pb2.AuthEnableRequest.FromString,
          response_serializer=rpc__pb2.AuthEnableResponse.SerializeToString,
      ),
      'AuthDisable': grpc.unary_unary_rpc_method_handler(
          servicer.AuthDisable,
          request_deserializer=rpc__pb2.AuthDisableRequest.FromString,
          response_serializer=rpc__pb2.AuthDisableResponse.SerializeToString,
      ),
      'Authenticate': grpc.unary_unary_rpc_method_handler(
          servicer.Authenticate,
          request_deserializer=rpc__pb2.AuthenticateRequest.FromString,
          response_serializer=rpc__pb2.AuthenticateResponse.SerializeToString,
      ),
      'UserAdd': grpc.unary_unary_rpc_method_handler(
          servicer.UserAdd,
          request_deserializer=rpc__pb2.AuthUserAddRequest.FromString,
          response_serializer=rpc__pb2.AuthUserAddResponse.SerializeToString,
      ),
      'UserGet': grpc.unary_unary_rpc_method_handler(
          servicer.UserGet,
          request_deserializer=rpc__pb2.AuthUserGetRequest.FromString,
          response_serializer=rpc__pb2.AuthUserGetResponse.SerializeToString,
      ),
      'UserList': grpc.unary_unary_rpc_method_handler(
          servicer.UserList,
          request_deserializer=rpc__pb2.AuthUserListRequest.FromString,
          response_serializer=rpc__pb2.AuthUserListResponse.SerializeToString,
      ),
      'UserDelete': grpc.unary_unary_rpc_method_handler(
          servicer.UserDelete,
          request_deserializer=rpc__pb2.AuthUserDeleteRequest.FromString,
          response_serializer=rpc__pb2.AuthUserDeleteResponse.SerializeToString,
      ),
      'UserChangePassword': grpc.unary_unary_rpc_method_handler(
          servicer.UserChangePassword,
          request_deserializer=rpc__pb2.AuthUserChangePasswordRequest.FromString,
          response_serializer=rpc__pb2.AuthUserChangePasswordResponse.SerializeToString,
      ),
      'UserGrantRole': grpc.unary_unary_rpc_method_handler(
          servicer.UserGrantRole,
          request_deserializer=rpc__pb2.AuthUserGrantRoleRequest.FromString,
          response_serializer=rpc__pb2.AuthUserGrantRoleResponse.SerializeToString,
      ),
      'UserRevokeRole': grpc.unary_unary_rpc_method_handler(
          servicer.UserRevokeRole,
          request_deserializer=rpc__pb2.AuthUserRevokeRoleRequest.FromString,
          response_serializer=rpc__pb2.AuthUserRevokeRoleResponse.SerializeToString,
      ),
      'RoleAdd': grpc.unary_unary_rpc_method_handler(
          servicer.RoleAdd,
          request_deserializer=rpc__pb2.AuthRoleAddRequest.FromString,
          response_serializer=rpc__pb2.AuthRoleAddResponse.SerializeToString,
      ),
      'RoleGet': grpc.unary_unary_rpc_method_handler(
          servicer.RoleGet,
          request_deserializer=rpc__pb2.AuthRoleGetRequest.FromString,
          response_serializer=rpc__pb2.AuthRoleGetResponse.SerializeToString,
      ),
      'RoleList': grpc.unary_unary_rpc_method_handler(
          servicer.RoleList,
          request_deserializer=rpc__pb2.AuthRoleListRequest.FromString,
          response_serializer=rpc__pb2.AuthRoleListResponse.SerializeToString,
      ),
      'RoleDelete': grpc.unary_unary_rpc_method_handler(
          servicer.RoleDelete,
          request_deserializer=rpc__pb2.AuthRoleDeleteRequest.FromString,
          response_serializer=rpc__pb2.AuthRoleDeleteResponse.SerializeToString,
      ),
      'RoleGrantPermission': grpc.unary_unary_rpc_method_handler(
          servicer.RoleGrantPermission,
          request_deserializer=rpc__pb2.AuthRoleGrantPermissionRequest.FromString,
          response_serializer=rpc__pb2.AuthRoleGrantPermissionResponse.SerializeToString,
      ),
      'RoleRevokePermission': grpc.unary_unary_rpc_method_handler(
          servicer.RoleRevokePermission,
          request_deserializer=rpc__pb2.AuthRoleRevokePermissionRequest.FromString,
          response_serializer=rpc__pb2.AuthRoleRevokePermissionResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'etcdserverpb.Auth', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))
